@@ -32,6 +32,7 @@ function printHelp()
   echo "                defaults to ../"                     1>&2
   echo "  --output PATH the directory where the source rpm"  1>&2
   echo "                should be stored, defaulting to ."   1>&2
+  echo "  --version VERSION the version provided by user"    1>&2
 }
 
 #-------------------------------------------------------------------------------
@@ -58,6 +59,13 @@ while test ${#} -ne 0; do
     fi
     OUTPUTPATH=${2}
     shift
+  elif test x${1} = x--version; then
+    if test ${#} -lt 2; then
+      echo "--version parameter needs an argument" 1>&2
+      exit 1
+    fi
+    USER_VERSION="--version ${2}"
+    shift
   fi
   shift
 done
@@ -112,7 +120,7 @@ if test ! -x $SOURCEPATH/genversion.sh; then
   exit 3
 fi
 
-VERSION=`$SOURCEPATH/genversion.sh --print-only $SOURCEPATH 2>/dev/null`
+VERSION=`$SOURCEPATH/genversion.sh --print-only $USER_VERSION $SOURCEPATH 2>/dev/null`
 if test $? -ne 0; then
   echo "[!] Unable to figure out the version number" 1>&2
   exit 4
@@ -141,6 +149,17 @@ if test x`echo $VERSION | egrep $CERNEXP` != x; then
   VERSION=`echo $VERSION | sed 's/-.*\.CERN//'`
 fi
 
+#-------------------------------------------------------------------------------
+# In case of user version check if the release number has been provided
+#-------------------------------------------------------------------------------
+if test x"$USER_VERSION" != x; then
+  TMP=`echo $VERSION | sed 's#.*-##g'`
+  if test $TMP != $VERSION; then
+    RELEASE=$TMP
+    VERSION=`echo $VERSION | sed 's#-[^-]*$##'`
+  fi
+fi
+
 VERSION=`echo $VERSION | sed 's/-/./g'`
 echo "[i] RPM compliant version: $VERSION-$RELEASE"
 
