@@ -405,41 +405,11 @@ int XrdConfig::Configure(int argc, char **argv)
 #endif
    }
 
-// Bind the log file if we have one
-//
-   if (LogInfo.logArg)
-      {LogInfo.xrdEnv = &theEnv;
-       LogInfo.iName  = myInsName;
-       LogInfo.cfgFn  = ConfigFN;
-       if (!XrdOucLogging::configLog(Log, LogInfo)) _exit(16);
-       Log.logger()->AddMsg(XrdBANNER);
-      }
-
-// Get the full host name. In theory, we should always get some kind of name.
-// We must define myIPAddr here because we may need to run in v4 mode and
-// that doesn't get set until after the options are scanned.
+// Get the full host name. We must define myIPAddr here because we may need to
+// run in v4 mode and that doesn't get set until after the options are scanned.
 //
    static XrdNetAddr *myIPAddr = new XrdNetAddr((int)0);
-   if (!(myName = myIPAddr->Name(0, &temp)))
-      {Log.Emsg("Config", "Unable to determine host name; ",
-                           (temp ? temp : "reason unknown"),
-                           "; execution terminated.");
-       _exit(16);
-      }
-
-// Verify that we have a real name. We've had problems with people setting up
-// bad /etc/hosts files that can cause connection failures if "allow" is used.
-// Otherwise, determine our domain name.
-//
-   if (!myIPAddr->isRegistered())
-      {Log.Emsg("Config",myName,"does not appear to be registered in the DNS.");
-       Log.Emsg("Config","Verify that the '/etc/hosts' file is correct and "
-                         "this machine is registered in DNS.");
-       Log.Emsg("Config", "Execution continues but connection failures may occur.");
-       myDomain = 0;
-      } else if (!(myDomain = index(myName, '.')))
-                Log.Say("Config warning: this hostname, ", myName,
-                            ", is registered without a domain qualification.");
+   if (!(myName = myIPAddr->Name(0, &temp))) myName = "";
 
 // Get our IP address and FQN
 //
@@ -459,6 +429,30 @@ int XrdConfig::Configure(int argc, char **argv)
    XrdOucEnv::Export("XRDHOST", myName);
    XrdOucEnv::Export("XRDNAME", ProtInfo.myInst);
    XrdOucEnv::Export("XRDPROG", myProg);
+
+// Bind the log file if we have one
+//
+   if (LogInfo.logArg)
+      {LogInfo.xrdEnv = &theEnv;
+       LogInfo.iName  = myInsName;
+       LogInfo.cfgFn  = ConfigFN;
+       if (!XrdOucLogging::configLog(Log, LogInfo)) _exit(16);
+       Log.logger()->AddMsg(XrdBANNER);
+      }
+
+// We now test for host name. In theory, we should always get some kind of name.
+// We can't really continue without some kind of name at this point. Note that
+// vriable temp should still be valid from the previous NetAddr call.
+//
+   if (!(*myName))
+      {Log.Emsg("Config", "Unable to determine host name; ",
+                           (temp ? temp : "reason unknown"),
+                           "; execution terminated.");
+       _exit(16);
+      }
+
+// Tell NetIF what logger to use as it's been properly setup by now.
+//
    XrdNetIF::SetMsgs(&Log);
 
 // Put out the herald
@@ -469,6 +463,20 @@ int XrdConfig::Configure(int argc, char **argv)
    Log.Say(0, buff);
    Log.Say(XrdBANNER);
 
+// Verify that we have a real name. We've had problems with people setting up
+// bad /etc/hosts files that can cause connection failures if "allow" is used.
+// Otherwise, determine our domain name.
+//
+   if (!myIPAddr->isRegistered())
+      {Log.Emsg("Config",myName,"does not appear to be registered in the DNS.");
+       Log.Emsg("Config","Verify that the '/etc/hosts' file is correct and "
+                         "this machine is registered in DNS.");
+       Log.Emsg("Config", "Execution continues but connection failures may occur.");
+       myDomain = 0;
+      } else if (!(myDomain = index(myName, '.')))
+                Log.Say("Config warning: this hostname, ", myName,
+                            ", is registered without a domain qualification.");
+
 // Setup the initial required protocol.
 //
    Firstcp = Lastcp = new XrdConfigProt(strdup(dfltProt), libProt, 0);
@@ -990,7 +998,7 @@ int XrdConfig::Setup(char *dfltp)
    if (PortWAN &&  (NetWAN = new XrdInet(&Log, &Trace, Police)))
       {if (Wan_Opts || Wan_Blen) NetWAN->setDefaults(Wan_Opts, Wan_Blen);
        if (myDomain) NetWAN->setDomain(myDomain);
-       if (NetWAN->Bind((PortWAN > 0 ? PortWAN : 0), "tcp")) return 1;
+       if (NetWAN->BindSD((PortWAN > 0 ? PortWAN : 0), "tcp")) return 1;
        PortWAN  = NetWAN->Port();
        wsz      = NetWAN->WSize();
        Wan_Blen = (wsz < Wan_Blen || !Wan_Blen ? wsz : Wan_Blen);
@@ -1012,7 +1020,7 @@ int XrdConfig::Setup(char *dfltp)
              if (Net_Opts || Net_Blen)
                 NetTCP[NetTCPlep]->setDefaults(Net_Opts, Net_Blen);
              if (myDomain) NetTCP[NetTCPlep]->setDomain(myDomain);
-             if (NetTCP[NetTCPlep]->Bind(cp->port, "tcp")) return 1;
+             if (NetTCP[NetTCPlep]->BindSD(cp->port, "tcp")) return 1;
              ProtInfo.Port   = NetTCP[NetTCPlep]->Port();
              ProtInfo.NetTCP = NetTCP[NetTCPlep];
              wsz             = NetTCP[NetTCPlep]->WSize();
@@ -1231,13 +1239,14 @@ int XrdConfig::xbuf(XrdSysError *eDest, XrdOucStream &Config)
 int XrdConfig::xnet(XrdSysError *eDest, XrdOucStream &Config)
 {
     char *val;
-    int  i, n, V_keep = -1, V_nodnr = 0, V_iswan = 0, V_blen = -1, V_ct = -1;
+    int  i, n, V_keep = -1, V_nodnr = 0, V_iswan = 0, V_blen = -1, V_ct = -1, V_assumev4;
     int  v_rpip = -1;
     long long llp;
     struct netopts {const char *opname; int hasarg; int opval;
                            int *oploc;  const char *etxt;}
            ntopts[] =
        {
+        {"assumev4",   0, 1, &V_assumev4, "option"},
         {"keepalive",  0, 1, &V_keep,   "option"},
         {"nokeepalive",0, 0, &V_keep,   "option"},
         {"kaparms",    4, 0, &V_keep,   "option"},
@@ -1320,6 +1329,7 @@ int XrdConfig::xnet(XrdSysError *eDest, XrdOucStream &Config)
 
      if (V_ct >= 0) XrdNetAddr::SetCache(V_ct);
      if (v_rpip >= 0) XrdInet::netIF.SetRPIPA(v_rpip != 0);
+     if (V_assumev4 >= 0) XrdInet::SetAssumeV4(true);
      return 0;
 }
 
