@@ -327,7 +327,7 @@ void undoProgBar(int isOK)
 /*                               c p F a t a l                                */
 /******************************************************************************/
   
-int cpFatal(const char *Act, XrdClient *cSrc, XrdClient *cDst)
+int cpFatal(const char *Act, XrdClient *cSrc, XrdClient *cDst, const char *hn=0)
 {
    XrdClient *cObj;
    const char *Msg;
@@ -337,7 +337,9 @@ int cpFatal(const char *Act, XrdClient *cSrc, XrdClient *cDst)
    if (cSrc) {cObj = cSrc; Msg = "Copy from ";}
       else   {cObj = cDst; Msg = "Copy to ";}
 
-   EMSG(Msg <<cObj->GetCurrentUrl().Host.c_str() <<" failed on " <<Act <<"!");
+   if (!hn) hn = cObj->GetCurrentUrl().Host.c_str();
+
+   EMSG(Msg <<hn <<" failed on " <<Act <<"!");
    EMSG(ServerError(cObj));
    return -1;
 }
@@ -720,12 +722,12 @@ int valTPC(XrdClient *cObj, int isDest)
   
 char *genDestCgi(XrdClient *xrdsrc, const char *src)
 {
-   union {int *intP;
+   union {long long intP;
           int  intV[2];
          } iKey;
    XrdClientStatInfo stat;
    XrdOucString dCGI;
-   int myKey[3], xTTL = 0;
+   int myKey[3];
    const char *Path, *cksVal, *cgiP;
    char *qP, aszBuff[128], lfnBuff[1032], cgiBuff[2048];
 
@@ -747,7 +749,7 @@ char *genDestCgi(XrdClient *xrdsrc, const char *src)
    gettimeofday(&abs_start_time,&tz);
    myKey[0] = abs_start_time.tv_usec;
    myKey[1] = getpid() | (getppid() << 16);
-   iKey.intP = &myKey[0];
+   iKey.intP = (long long) &myKey[0];
    myKey[2] = iKey.intV[0] ^ iKey.intV[1];
    sprintf(tpcKey, "%08x%08x%08x", myKey[0], myKey[1], myKey[2]);
 
@@ -796,10 +798,8 @@ int doCp_xrd3xrd(XrdClient *xrddest, const char *src, const char *dst)
    XrdClientUrlInfo dUrl;
    XrdOucString sUrl(tpcSrc->GetCurrentUrl().GetUrl().c_str());
    XrdOucString *rCGI, dstUrl;
-   pthread_t myTID;
    int xTTL = -1;
    const char *cgiP;
-   char *qP;
    char cgiBuff[1024];
 
 // Append any redirection cgi information to our source spec
@@ -838,9 +838,10 @@ int doCp_xrd3xrd(XrdClient *xrddest, const char *src, const char *dst)
 // Open the source
 //
    Client.Src = new XrdClient(sUrl.c_str());
+   const char *hName = Client.Src->GetCurrentUrl().Host.c_str();
    if ((!Client.Src->Open(0, kXR_async) ||
       (Client.Src->LastServerResp()->status != kXR_ok)))
-      return cpFatal("open", Client.Src, 0);
+      return cpFatal("open", Client.Src, 0, hName);
    
 // Start the progress bar if so wanted
 //
@@ -908,9 +909,10 @@ int doCp_xrd2xrd(XrdClient **xrddest, const char *src, const char *dst) {
    // If Xrdcli is non-null, the correct src file has already been opened
    if (!cpnfo.XrdCli)
       {cpnfo.XrdCli = new XrdClient(src);
+       const char *hName = cpnfo.XrdCli->GetCurrentUrl().Host.c_str();
        if ( ( !cpnfo.XrdCli->Open(0, kXR_async) ||
           (cpnfo.XrdCli->LastServerResp()->status != kXR_ok) ) )
-          {cpFatal("open", cpnfo.XrdCli, 0);
+          {cpFatal("open", cpnfo.XrdCli, 0, hName);
            delete cpnfo.XrdCli;
            cpnfo.XrdCli = 0;
            return 1;
@@ -925,9 +927,10 @@ int doCp_xrd2xrd(XrdClient **xrddest, const char *src, const char *dst) {
    // if xrddest if nonzero, then the file is already opened for writing
    if (!*xrddest) {
       *xrddest = new XrdClient(dest.c_str());
+       const char *hName = (*xrddest)->GetCurrentUrl().Host.c_str();
       
       if (!PedanticOpen4Write(*xrddest, rwMode, xrd_wr_flags))
-         {cpFatal("open", 0, *xrddest);
+         {cpFatal("open", 0, *xrddest, hName);
           delete cpnfo.XrdCli;
           delete *xrddest;
           *xrddest = 0;
@@ -1091,9 +1094,10 @@ int doCp_xrd2loc(const char *src, const char *dst) {
    // If Xrdcli is non-null, the correct src file has already been opened
    if (!cpnfo.XrdCli)
       {cpnfo.XrdCli = new XrdClient(src);
+       const char *hName = cpnfo.XrdCli->GetCurrentUrl().Host.c_str();
        if ( ( !cpnfo.XrdCli->Open(0, kXR_async) ||
           (cpnfo.XrdCli->LastServerResp()->status != kXR_ok) ) )
-          {cpFatal("open", cpnfo.XrdCli, 0);
+          {cpFatal("open", cpnfo.XrdCli, 0, hName);
            delete cpnfo.XrdCli;
            cpnfo.XrdCli = 0;
            return 1;
@@ -1298,8 +1302,9 @@ int doCp_loc2xrd(XrdClient **xrddest, const char *src, const char * dst) {
    // if xrddest if nonzero, then the file is already opened for writing
    if (!*xrddest)
       {*xrddest = new XrdClient(dest.c_str());
+       const char *hName = (*xrddest)->GetCurrentUrl().Host.c_str();
        if (!PedanticOpen4Write(*xrddest, rwMode, xrd_wr_flags) )
-          {cpFatal("open", 0 , *xrddest);
+          {cpFatal("open", 0 , *xrddest, hName);
            close(cpnfo.localfile);
            delete *xrddest;
            *xrddest = 0;
@@ -1454,7 +1459,7 @@ int doCp(XrdOucString &src, XrdOucString &dest, XrdClient *xrddest)
 int main(int argc, char**argv)
 {
    const char *Opaque;
-   char *srcpath = 0, *destpath = 0;
+   char *hName, *srcpath = 0, *destpath = 0;
 
 // Preset globals
 //
@@ -1471,7 +1476,7 @@ int main(int argc, char**argv)
 // Invoke config; it it returns then all went well.
 //
    Config.Config(argc, argv, XrdCpConfig::opt1Src|XrdCpConfig::optNoStdIn
-                            |XrdCpConfig::optNoXtnd);
+                            |XrdCpConfig::optNoXtnd|XrdCpConfig::optNoLclCp);
 
 // Turn off any blab from the client
 //
@@ -1556,7 +1561,7 @@ int main(int argc, char**argv)
 // Establish checksum processing
 //
    setCks = 0;
-   lclCks = Config.Want(XrdCpConfig::DoAdler)||Config.Want(XrdCpConfig::DoMD5);
+   lclCks = Config.Want(XrdCpConfig::DoCksrc);
    xeqCks = Config.Want(XrdCpConfig::DoCksum);
    prtCks =(xeqCks || lclCks) &&  Config.Want(XrdCpConfig::DoCkprt);
    getCks = xeqCks && (Config.CksData.Length == 0);
@@ -1573,6 +1578,13 @@ int main(int argc, char**argv)
  //    if (dup2(STDOUT_FILENO, STDERR_FILENO)) cerr <<"??? " <<errno <<endl;
       }
 
+// Extract source host if present
+//
+   if (strncmp(srcpath, "root://", 7) || strncmp(srcpath, "xroot://", 8) )
+      {XrdClientUrlInfo sUrl(srcpath);
+       hName = (sUrl.IsValid() ? strdup(sUrl.Host.c_str()) : 0);
+      } else hName = 0;
+
 // Prepare to generate a copy list
 //
    XrdCpWorkLst *wklst = new XrdCpWorkLst();
@@ -1583,7 +1595,7 @@ int main(int argc, char**argv)
 // Generate the sources
 //
    if (wklst->SetSrc(&cpnfo.XrdCli, srcpath, Config.srcOpq, recurse, 1))
-      {cpFatal("open", cpnfo.XrdCli, 0);
+      {cpFatal("open", cpnfo.XrdCli, 0, hName);
        exit(1);
       }
 
@@ -1594,12 +1606,21 @@ int main(int argc, char**argv)
             tpcSrc = cpnfo.XrdCli;
            }
 
+// Extract source host if present
+//
+   if (hName) free(hName);
+   if (strncmp(destpath, "root://", 7) || strncmp(destpath, "xroot://", 8) )
+      {XrdClientUrlInfo dUrl(destpath);
+       hName = (dUrl.IsValid() ? strdup(dUrl.Host.c_str()) : 0);
+      } else hName = 0;
+
 // Verify the correctness of the destination
 //
    if (wklst->SetDest(&xrddest, destpath, Opaque, xrd_wr_flags, 1))
-      {cpFatal("open", 0, xrddest);
+      {cpFatal("open", 0, xrddest, hName);
        exit(1);
       }
+   if (hName) {free(hName); hName = 0;}
 
       // Initialize monitoring client, if a plugin is present
       cpnfo.mon = 0;
