@@ -21,6 +21,8 @@
 #include "XrdCl/XrdClLog.hh"
 #include "XrdCl/XrdClMessage.hh"
 #include "XrdCl/XrdClAsyncSocketHandler.hh"
+#include "XrdCl/XrdClXRootDTransport.hh"
+#include "XrdCl/XrdClOptimizers.hh"
 #include <netinet/tcp.h>
 
 namespace XrdCl
@@ -41,6 +43,7 @@ namespace XrdCl
     pIncoming( 0 ),
     pHSIncoming( 0 ),
     pOutgoing( 0 ),
+    pSignature( 0 ),
     pHSOutgoing( 0 ),
     pHandShakeData( 0 ),
     pHandShakeDone( false ),
@@ -71,6 +74,7 @@ namespace XrdCl
   {
     Close();
     delete pSocket;
+    delete pSignature;
   }
 
   //----------------------------------------------------------------------------
@@ -371,6 +375,17 @@ namespace XrdCl
 
       pOutgoing->SetCursor( 0 );
       pOutMsgSize = pOutgoing->GetSize();
+
+      //------------------------------------------------------------------------
+      // Secure the message if necessary
+      //------------------------------------------------------------------------
+      delete pSignature; pSignature = 0;
+      XRootDStatus st = GetSignature( pOutgoing, pSignature );
+      if( !st.IsOK() )
+      {
+        OnFault( st );
+        return;
+      }
     }
 
     //--------------------------------------------------------------------------
@@ -379,7 +394,7 @@ namespace XrdCl
     Status st;
     if( !pOutMsgDone )
     {
-      if( !(st = WriteCurrentMessage( pOutgoing )).IsOK() )
+      if( !(st = WriteSignedMessage( pOutgoing, pSignature )).IsOK() )
       {
         OnFault( st );
         return;
@@ -505,6 +520,56 @@ namespace XrdCl
     return Status();
   }
 
+  //----------------------------------------------------------------------------
+  // Write the message and its signature
+  //----------------------------------------------------------------------------
+  Status AsyncSocketHandler::WriteSignedMessage( Message *toWrite, Message *&sign )
+  {
+    if( !sign ) return WriteCurrentMessage( toWrite );
+
+    Log *log = DefaultEnv::GetLog();
+
+    const size_t iovcnt = 2;
+    iovec iov[iovcnt];
+    ToIov( *sign,    iov[0] );
+    ToIov( *toWrite, iov[1] );
+
+    uint32_t  leftToBeWritten = iov[0].iov_len + iov[1].iov_len;
+
+    while( leftToBeWritten )
+    {
+      int bytesRead = pSocket->WriteV( iov, iovcnt );
+      if( bytesRead <= 0 )
+      {
+        //----------------------------------------------------------------------
+        // Writing operation would block! So we are done for now, but we will
+        // return
+        //----------------------------------------------------------------------
+        if( errno == EAGAIN || errno == EWOULDBLOCK )
+          return Status( stOK, suRetry );
+
+        //----------------------------------------------------------------------
+        // Actual socket error error!
+        //----------------------------------------------------------------------
+        sign->SetCursor( 0 );
+        toWrite->SetCursor( 0 );
+        return Status( stError, errSocketError, errno );
+      }
+
+      leftToBeWritten -= bytesRead;
+      UpdateAfterWrite( *sign,    iov[0], bytesRead );
+      UpdateAfterWrite( *toWrite, iov[1], bytesRead );
+    }
+
+    //--------------------------------------------------------------------------
+    // We have written the message successfully
+    //--------------------------------------------------------------------------
+    log->Dump( AsyncSockMsg, "[%s] Wrote a message: %s (0x%x), %d bytes",
+               pStreamName.c_str(), toWrite->GetDescription().c_str(),
+               toWrite, toWrite->GetSize() );
+    return Status();
+  }
+
   //----------------------------------------------------------------------------
   // Got a read readiness event
   //----------------------------------------------------------------------------
@@ -756,7 +821,19 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void AsyncSocketHandler::OnReadTimeout()
   {
-    pStream->OnReadTimeout( pSubStreamNum );
+    bool isBroken = false;
+    pStream->OnReadTimeout( pSubStreamNum, isBroken );
+
+    if( isBroken )
+    {
+      // if we are here it means Stream::OnError has been
+      // called from inside of Stream::OnReadTimeout, this
+      // in turn means that the ownership of following
+      // pointers, has been transfered to the inQueue
+      pIncoming   = 0;
+      pOutgoing   = 0;
+      pOutHandler = 0;
+    }
   }
 
   //----------------------------------------------------------------------------
@@ -768,4 +845,37 @@ namespace XrdCl
     if( now > pConnectionStarted+pConnectionTimeout )
       OnFaultWhileHandshaking( Status( stError, errSocketTimeout ) );
   }
+
+  //------------------------------------------------------------------------
+  // Get signature for given message
+  //------------------------------------------------------------------------
+  Status AsyncSocketHandler::GetSignature( Message *toSign, Message *&sign )
+  {
+    // ideally the 'GetSignature' method should be in  TransportHandler interface
+    // however due to ABI compatibility for the time being this workaround has to
+    // be employed
+    XRootDTransport *xrootdTransport = dynamic_cast<XRootDTransport*>( pTransport );
+    if( !xrootdTransport ) return Status( stError, errNotSupported );
+    return xrootdTransport->GetSignature( toSign, sign, *pChannelData );
+  }
+
+  //------------------------------------------------------------------------
+  // Initialize the iovec with given message
+  //------------------------------------------------------------------------
+  void AsyncSocketHandler::ToIov( Message &msg, iovec &iov )
+  {
+    iov.iov_base = msg.GetBufferAtCursor();
+    iov.iov_len  = msg.GetSize() - msg.GetCursor();
+  }
+
+  //------------------------------------------------------------------------
+  // Update iovec after write
+  //------------------------------------------------------------------------
+  void AsyncSocketHandler::UpdateAfterWrite( Message &msg, iovec &iov, int &bytesRead )
+  {
+    size_t advance = ( bytesRead < (int)iov.iov_len ) ? bytesRead : iov.iov_len;
+    bytesRead -= advance;
+    msg.AdvanceCursor( advance );
+    ToIov( msg, iov );
+  }
 }
