@@ -306,12 +306,6 @@ bool AllOptionsSupported( XrdCpConfig *config )
     return false;
   }
 
-  if( config->nSrcs != 1 )
-  {
-    std::cerr << "Multiple sources are not yet supported" << std::endl;
-    return false;
-  }
-
   return true;
 }
 
@@ -601,6 +595,27 @@ int main( int argc, char **argv )
     }
   }
 
+  //----------------------------------------------------------------------------
+  // ZIP archive
+  //----------------------------------------------------------------------------
+  std::string zipFile;
+  bool        zip = false;
+  if( config.Want( XrdCpConfig::DoZip ) )
+  {
+    zipFile = config.zipFile;
+    zip = true;
+  }
+
+  //----------------------------------------------------------------------------
+  // Extreme Copy
+  //----------------------------------------------------------------------------
+  int nbSources = 0;
+  bool xcp      = false;
+  if( config.Want( XrdCpConfig::DoSources ) )
+  {
+    nbSources = config.nSrcs;
+    xcp       = true;
+  }
 
   //----------------------------------------------------------------------------
   // Environment settings
@@ -612,6 +627,9 @@ int main( int argc, char **argv )
   int chunkSize = DefaultCPChunkSize;
   env->GetInt( "CPChunkSize", chunkSize );
 
+  int blockSize = DefaultXCpBlockSize;
+  env->GetInt( "XCpBlockSize", blockSize );
+
   int parallelChunks = DefaultCPParallelChunks;
   env->GetInt( "CPParallelChunks", parallelChunks );
   if( parallelChunks < 1 ||
@@ -756,7 +774,11 @@ int main( int argc, char **argv )
     {
       target = dest + "/";
       // if it is a metalink we don't want to use the metalink name
-      if( src.IsMetalink() )
+      if( zip )
+      {
+        target += zipFile;
+      }
+      else if( src.IsMetalink() )
       {
         XrdCl::RedirectorRegistry &registry = XrdCl::RedirectorRegistry::Instance();
         VirtualRedirector *redirector = registry.Get( source );
@@ -780,6 +802,16 @@ int main( int argc, char **argv )
     properties.Set( "checkSumPreset", checkSumPreset );
     properties.Set( "chunkSize",      chunkSize      );
     properties.Set( "parallelChunks", parallelChunks );
+    properties.Set( "zipArchive",     zip            );
+    properties.Set( "xcp",            xcp            );
+    properties.Set( "xcpBlockSize",   blockSize      );
+
+    if( zip )
+      properties.Set( "zipSource",    zipFile        );
+
+    if( xcp )
+      properties.Set( "nbXcpSources", nbSources      );
+
 
     XRootDStatus st = process.AddJob( properties, results );
     if( !st.IsOK() )
