@@ -27,6 +27,7 @@
 #include "XrdCl/XrdClCheckSumManager.hh"
 #include "XrdCl/XrdClTransportManager.hh"
 #include "XrdCl/XrdClPlugInManager.hh"
+#include "XrdCl/XrdClOptimizers.hh"
 #include "XrdOuc/XrdOucPreload.hh"
 #include "XrdSys/XrdSysAtomics.hh"
 #include "XrdSys/XrdSysUtils.hh"
@@ -285,6 +286,8 @@ namespace XrdCl
     REGISTER_VAR_INT( varsInt, "ParallelEvtLoop",      DefaultParallelEvtLoop      );
     REGISTER_VAR_INT( varsInt, "MetalinkProcessing",   DefaultMetalinkProcessing   );
     REGISTER_VAR_INT( varsInt, "LocalMetalinkFile",    DefaultLocalMetalinkFile    );
+    REGISTER_VAR_INT( varsInt, "XCpBlockSize",         DefaultXCpBlockSize         );
+    REGISTER_VAR_INT( varsInt, "NoDelay",              DefaultNoDelay              );
 
     REGISTER_VAR_STR( varsStr, "PollerPreference",     DefaultPollerPreference     );
     REGISTER_VAR_STR( varsStr, "ClientMonitor",        DefaultClientMonitor        );
@@ -308,14 +311,19 @@ namespace XrdCl
 
     XrdSysPwd pwdHandler;
     passwd *pwd = pwdHandler.Get( getuid() );
-    std::string userConfigFile = pwd->pw_dir;
-    userConfigFile += "/.xrootd/client.conf";
+    if( pwd )
+    {
+      std::string userConfigFile = pwd->pw_dir;
+      userConfigFile += "/.xrootd/client.conf";
 
-    st = Utils::ProcessConfig( userConfig, userConfigFile );
+      st = Utils::ProcessConfig( userConfig, userConfigFile );
 
-    if( !st.IsOK() )
-      log->Debug( UtilityMsg, "Unable to process user config file: %s",
-                  st.ToString().c_str() );
+      if( !st.IsOK() )
+        log->Debug( UtilityMsg, "Unable to process user config file: %s",
+                    st.ToString().c_str() );
+    }
+    else
+      log->Debug( UtilityMsg, "Unable to find user home directory." );
 
     std::map<std::string, std::string>::iterator it;
 
@@ -663,7 +671,6 @@ namespace XrdCl
       "libXrdSecgsi.so",
       "libXrdSecgsiAuthzVO.so",
       "libXrdSecgsiGMAPDN.so",
-      "libXrdSecgsiGMAPLDAP.so",
       "libXrdSecpwd.so",
       "libXrdSecsss.so",
       "libXrdSecunix.so",
