@@ -253,7 +253,7 @@ namespace XrdCl
           pHandler = 0;
         }
         // and than destroy myself if this is the last reference
-        DestroyMyself();
+        DestroyMyself( scopedLock );
       }
       //------------------------------------------------------------------------
       //! Increment reference counter
@@ -281,20 +281,23 @@ namespace XrdCl
           pHandler = 0;
         }
         // destroy the object if it is
-        DestroyMyself();
+        DestroyMyself( scopedLock );
       }
 
     private:
       //------------------------------------------------------------------------
       //! Deletes itself only if this is the last reference
       //------------------------------------------------------------------------
-      void DestroyMyself()
+      void DestroyMyself( XrdSysMutexHelper &lck )
       {
         // decrement the reference counter
         --pReferenceCounter;
         // if the object is not used anymore delete it
         if( pReferenceCounter == 0)
+        {
+          lck.UnLock();
           delete this;
+        }
       }
       //------------------------------------------------------------------------
       //! Private Destructor (use 'Destroy' method)
@@ -390,7 +393,9 @@ namespace XrdCl
       ResetMonitoringVars();
     }
 
-    if( pUseVirtRedirector && pFileUrl && pFileUrl->IsMetalink() )
+    // check if the logger is still there, this is only for root, as root might
+    // have unload us already so in this case we don't want to do anything
+    if( DefaultEnv::GetLog() && pUseVirtRedirector && pFileUrl && pFileUrl->IsMetalink() )
     {
       RedirectorRegistry& registry = RedirectorRegistry::Instance();
       registry.Release( *pFileUrl );
@@ -427,7 +432,7 @@ namespace XrdCl
         pFileState == Recovering )
       return XRootDStatus( stError, errInvalidOp );
 
-    pStatus = OpenInProgress;
+    pFileState = OpenInProgress;
 
     //--------------------------------------------------------------------------
     // Check if the parameters are valid
@@ -550,7 +555,7 @@ namespace XrdCl
         pFileState == Recovering || !pInTheFly.empty() )
       return XRootDStatus( stError, errInvalidOp );
 
-    pStatus = CloseInProgress;
+    pFileState = CloseInProgress;
 
     Log *log = DefaultEnv::GetLog();
     log->Debug( FileMsg, "[0x%x@%s] Sending a close command for handle 0x%x to "
@@ -1352,8 +1357,10 @@ namespace XrdCl
   //------------------------------------------------------------------------
   void FileStateHandler::Tick( time_t now )
   {
-    XrdSysMutexHelper scopedLock( pMutex );
-    TimeOutRequests( now );
+    if (pMutex.CondLock())
+       {TimeOutRequests( now );
+        pMutex.UnLock();
+       }
   }
 
   //----------------------------------------------------------------------------
@@ -1668,9 +1675,14 @@ namespace XrdCl
                   rd.request->GetDescription().c_str() );
       return;
     }
+
+    JobManager *jobMan = DefaultEnv::GetPostMaster()->GetJobManager();
     ResponseHandler *userHandler = sh->GetUserHandler();
-    userHandler->HandleResponseWithHosts( new XRootDStatus( status ), 0,
-                                          rd.params.hostList );
+    jobMan->QueueJob( new ResponseJob(
+                        userHandler,
+                        new XRootDStatus( status ),
+                        0, rd.params.hostList ) );
+
     delete sh;
   }
 
