@@ -32,6 +32,7 @@
 #include "XrdCl/XrdClUglyHacks.hh"
 #include "XrdCl/XrdClUtils.hh"
 #include "XrdCl/XrdClTaskManager.hh"
+#include "XrdCl/XrdClJobManager.hh"
 #include "XrdCl/XrdClSIDManager.hh"
 #include "XrdCl/XrdClMessageUtils.hh"
 
@@ -64,10 +65,37 @@ namespace
     private:
       XrdCl::XRootDMsgHandler *pHandler;
   };
+
 };
 
 namespace XrdCl
 {
+
+  //----------------------------------------------------------------------------
+  // Delegate the response handling to the thread-pool
+  //----------------------------------------------------------------------------
+  class HandleRspJob: public XrdCl::Job
+  {
+    public:
+      HandleRspJob( XrdCl::XRootDMsgHandler *handler ): pHandler( handler )
+      {
+
+      }
+
+      virtual ~HandleRspJob()
+      {
+
+      }
+
+      virtual void Run( void *arg )
+      {
+        pHandler->HandleResponse();
+        delete this;
+      }
+    private:
+      XrdCl::XRootDMsgHandler *pHandler;
+  };
+
   //----------------------------------------------------------------------------
   // Examine an incoming message, and decide on the action to be taken
   //----------------------------------------------------------------------------
@@ -386,9 +414,12 @@ namespace XrdCl
         //----------------------------------------------------------------------
         if( !pRedirectCounter )
         {
-          log->Dump( XRootDMsg, "[%s] Redirect limit has been reached for"
-                     "message %s", pUrl.GetHostId().c_str(),
-                     pRequest->GetDescription().c_str() );
+          log->Warning( XRootDMsg, "[%s] Redirect limit has been reached for "
+                     "message %s, the last known error is: %s",
+                     pUrl.GetHostId().c_str(),
+                     pRequest->GetDescription().c_str(),
+                     pLastError.ToString().c_str() );
+
 
           pStatus = Status( stFatal, errRedirectLimit );
           HandleResponse();
@@ -1804,6 +1835,8 @@ namespace XrdCl
     if( status.IsOK() )
       return;
 
+    pLastError = status;
+
     Log *log = DefaultEnv::GetLog();
     log->Debug( XRootDMsg, "[%s] Handling error while processing %s: %s.",
                 pUrl.GetHostId().c_str(), pRequest->GetDescription().c_str(),
@@ -1835,7 +1868,7 @@ namespace XrdCl
       else
       {
         pStatus = status;
-        HandleResponse();
+        HandleRspOrQueue();
         return;
       }
     }
@@ -1853,7 +1886,7 @@ namespace XrdCl
                   pUrl.GetHostId().c_str(),
                   pRequest->GetDescription().c_str() );
       pStatus = status;
-      HandleResponse();
+      HandleRspOrQueue();
       return;
     }
 
@@ -1877,7 +1910,7 @@ namespace XrdCl
         return;
       }
       pStatus = status;
-      HandleResponse();
+      HandleRspOrQueue();
       return;
     }
   }
@@ -1887,8 +1920,44 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   Status XRootDMsgHandler::RetryAtServer( const URL &url )
   {
+    Log *log = DefaultEnv::GetLog();
+
     if( pUrl.GetHostId() != url.GetHostId() )
+    {
       pHosts->push_back( url );
+
+      //------------------------------------------------------------------------
+      // Assign a new stream id to the message
+      //------------------------------------------------------------------------
+
+      // first release the old stream id
+      ClientRequestHdr *req = (ClientRequestHdr*)pRequest->GetBuffer();
+      pSidMgr->ReleaseSID( req->streamid );
+      pSidMgr = 0;
+
+      // then get the new SIDManager
+      AnyObject sidMgrObj;
+      Status st = pPostMaster->QueryTransport( url, XRootDQuery::SIDManager,
+                                              sidMgrObj );
+      if( !st.IsOK() )
+      {
+        log->Error( XRootDMsg, "[%s] Impossible to send message %s.",
+        pUrl.GetHostId().c_str(),
+        pRequest->GetDescription().c_str() );
+        return st;
+      }
+      sidMgrObj.Get( pSidMgr );
+
+      // and finally allocate new stream id
+      st = pSidMgr->AllocateSID( req->streamid );
+      if( !st.IsOK() )
+      {
+        log->Error( XRootDMsg, "[%s] Impossible to send message %s.",
+        pUrl.GetHostId().c_str(),
+        pRequest->GetDescription().c_str() );
+        return st;
+      }
+    }
     pUrl = url;
     return pPostMaster->Send( pUrl, pRequest, this, true, pExpiration );
   }
@@ -1960,4 +2029,17 @@ namespace XrdCl
     XRootDTransport::SetDescription( pRequest );
     XRootDTransport::MarshallRequest( pRequest );
   }
+
+  //------------------------------------------------------------------------
+  // If the current thread is a worker thread from our thread-pool
+  // handle the response, otherwise submit a new task to the thread-pool
+  //------------------------------------------------------------------------
+  void XRootDMsgHandler::HandleRspOrQueue()
+  {
+    JobManager *jobMgr = pPostMaster->GetJobManager();
+    if( jobMgr->IsWorker() )
+      HandleResponse();
+    else
+      jobMgr->QueueJob( new HandleRspJob( this ), 0 );
+  }
 }
