@@ -354,22 +354,54 @@ static int xrootdfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     }
 }
 
-static int xrootdfs_mknod(const char *path, mode_t mode, dev_t rdev)
+static int xrootdfs_do_create(const char *path, const char *url, int oflags, bool use_link_id, int *fd)
+/* Actually implement creating a file as generally as possible.
+ *
+ * path:        file path
+ * url:         xrootd root url to use
+ * oflags:      posix oflags
+ * use_link_id: register multiple secsss user numbers for editurl
+ * fd:          return file descriptor here
+ *
+ * returns:     0 on success, -errno on error.
+ */
 {
-    int res;
+    int res, link_id;
+    int *p_link_id = NULL;
+    char rootpath[MAXROOTURLLEN] = "";
 
-    /* On Linux this could just be 'mknod(path, mode, rdev)' but this
-       is more portable */
-    char rootpath[MAXROOTURLLEN];
+    if (use_link_id)
+        p_link_id = &link_id;
 
-    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, 0);
-    if (S_ISREG(mode))
-    {
-        rootpath[0]='\0';
-        strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
-        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
+    XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, p_link_id);
+    strncat(rootpath, url, MAXROOTURLLEN - strlen(rootpath) - 1);
+    strncat(rootpath, path, MAXROOTURLLEN - strlen(rootpath) - 1);
 
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
+    XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, p_link_id);
+    res = XrdFfsPosix_open(rootpath, oflags, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+    if (res == -1)
+        return -errno;
+    if (fd != NULL)
+        *fd = res;
+    return 0;
+}
+
+static int xrootdfs_mknod(const char *path, mode_t mode, dev_t rdev)
+/* Implement MKNOD for FUSE, which is supposed to be used for special files.
+ * Can be used when CREATE is not available.
+ * This implementation only supports normal files.
+ * Just create the file, then close it. Also create cns file.
+ *
+ * path:        file path
+ * mode:        file attribute bitmask
+ * rdev:        device number
+ *
+ * returns:     0 on success, -errno on error.
+ */
+{
+    int res, fd;
+    if (!S_ISREG(mode))
+        return -EPERM;
 /* 
    Around May 2008, the O_EXCL was added to the _open(). No reason was given. It is removed again 
    due to the following reason (the situation that redirector thinks a file exist while it doesn't):
@@ -383,25 +415,39 @@ static int xrootdfs_mknod(const char *path, mode_t mode, dev_t rdev)
 
         res = XrdFfsPosix_open(rootpath, O_CREAT | O_EXCL | O_WRONLY, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH); 
 */
-        res = XrdFfsPosix_open(rootpath, O_CREAT | O_WRONLY, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH); 
-        if (res == -1)
-            return -errno;
-        XrdFfsPosix_close(res);
-/* We have to make sure CNS file is created as well, otherwise, xrootdfs_getattr()
-   may or may not find this file (due to multi-threads) */
-        if (xrootdfs.cns == NULL)
-            return 0;
-
-        rootpath[0]='\0';
-        strncat(rootpath,xrootdfs.cns, MAXROOTURLLEN - strlen(rootpath) -1);
-        strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
-
-        XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, 0);
-        res = XrdFfsPosix_open(rootpath, O_CREAT | O_EXCL, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH); 
-        XrdFfsPosix_close(res);
-/* We actually don't need to care about error by this XrdFfsPosix_open() */
+    res = xrootdfs_do_create(path, xrootdfs.rdr, O_CREAT | O_WRONLY, false, &fd);
+    XrdFfsPosix_close(fd);
+    if (xrootdfs.cns != NULL)
+    {
+        xrootdfs_do_create(path, xrootdfs.cns, O_CREAT | O_EXCL, false, &fd);
+        XrdFfsPosix_close(fd);
     }
-    return 0;
+    return res;
+}
+
+static int xrootdfs_create(const char *path, mode_t mode, struct fuse_file_info *fi)
+/* Implement CREATE for FUSE, to create and open a normal file.
+ * Create a file and return its fd. Also create cns file.
+ *
+ * path:        file path
+ * mode:        file attribute bitmask
+ * fi:          file info, return file descriptor in here
+ *
+ * returns:     0 on success, -errno on error.
+ */
+{
+    int res, fd;
+    if (!S_ISREG(mode))
+        return -EPERM;
+    res = xrootdfs_do_create(path, xrootdfs.rdr, O_CREAT | O_WRONLY, true, &fd);
+    fi->fh = fd;
+    XrdFfsWcache_create(fd);    // Unlike mknod and like open, prepare wcache.
+    if (xrootdfs.cns != NULL)
+    {
+        xrootdfs_do_create(path, xrootdfs.cns, O_CREAT | O_EXCL, false, &fd);
+        XrdFfsPosix_close(fd);
+    }
+    return res;
 }
 
 static int xrootdfs_mkdir(const char *path, mode_t mode)
@@ -715,21 +761,30 @@ static int xrootdfs_utimens(const char *path, const struct timespec ts[2])
 }
 
 static int xrootdfs_open(const char *path, struct fuse_file_info *fi)
+/*
+ * path:    path to file
+ * fi:      file info, return file descriptor in fi->fh
+ *
+ * returns 0 on success and -errno on error
+ */
 {
-    int res, lid = 1;
+    int fd, lid = 1;
     char rootpath[MAXROOTURLLEN]="";
     strncat(rootpath,xrootdfs.rdr, MAXROOTURLLEN - strlen(rootpath) -1);
     strncat(rootpath,path, MAXROOTURLLEN - strlen(rootpath) -1);
 
     XrdFfsMisc_xrd_secsss_register(fuse_get_context()->uid, fuse_get_context()->gid, &lid);
     XrdFfsMisc_xrd_secsss_editurl(rootpath, fuse_get_context()->uid, &lid);
-    res = XrdFfsPosix_open(rootpath, fi->flags, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
-    if (res == -1)
+    fd = XrdFfsPosix_open(rootpath, fi->flags, S_IRUSR|S_IWUSR|S_IRGRP|S_IROTH);
+    if (fd == -1)
         return -errno;
 
-    fi->fh = res;
-    XrdFfsWcache_create(fi->fh);
-    return 0;
+    fi->fh = fd;
+    // be careful, 0 means error for this function
+    if (XrdFfsWcache_create(fi->fh))
+        return 0;
+    else
+        return -errno;
 }
 
 static int xrootdfs_read(const char *path, char *buf, size_t size, off_t offset,
@@ -1206,6 +1261,7 @@ int main(int argc, char *argv[])
     xrootdfs_oper.readdir	= xrootdfs_readdir;
     xrootdfs_oper.mknod		= xrootdfs_mknod;
     xrootdfs_oper.mkdir		= xrootdfs_mkdir;
+    xrootdfs_oper.create	= xrootdfs_create;
     xrootdfs_oper.symlink	= xrootdfs_symlink;
     xrootdfs_oper.unlink	= xrootdfs_unlink;
     xrootdfs_oper.rmdir		= xrootdfs_rmdir;
@@ -1233,7 +1289,10 @@ int main(int argc, char *argv[])
     cmdline_opts = (char **) malloc(sizeof(char*) * (argc -1 + 3));
     cmdline_opts[0] = argv[0];
     cmdline_opts[1] = strdup("-o");
-    cmdline_opts[2] = strdup("fsname=xrootdfs,allow_other,max_write=131072,attr_timeout=10,entry_timeout=10,negative_timeout=5");
+    if (getenv("XROOTDFS_NO_ALLOW_OTHER") != NULL && ! strcmp(getenv("XROOTDFS_NO_ALLOW_OTHER"),"1") )
+        cmdline_opts[2] = strdup("fsname=xrootdfs,max_write=131072,attr_timeout=10,entry_timeout=10,negative_timeout=5");
+    else
+        cmdline_opts[2] = strdup("fsname=xrootdfs,allow_other,max_write=131072,attr_timeout=10,entry_timeout=10,negative_timeout=5");
 
     for (int i = 1; i < argc; i++)
         cmdline_opts[i+2] = argv[i];
