@@ -52,12 +52,6 @@
 #include "XrdSys/XrdSysPlatform.hh"
 #include "XrdSys/XrdSysError.hh"
 #include "XrdSys/XrdSysFD.hh"
-
-/******************************************************************************/
-/*                               G l o b a l s                                */
-/******************************************************************************/
-  
-#define XRDNET_UDPBUFFSZ 32768
   
 /******************************************************************************/
 /*                           C o n s t r u c t o r                            */
